package template;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.function.Function;

public class CuttingTemplate {

	private int width;
	private int height;
	private String name;
	
	// Saves lines in cutting template
	private ArrayList<Line> lines;

	public CuttingTemplate(int width, int height) {
		this.width = width;
		this.height = height;
		// Create new empty ArrayList to Variable lines
		this.lines = new ArrayList<Line>();
	}

	/**
	 * @return width (in px) of the template
	 */
	public int getWidth() {
		return width;
	}

	/**
	 * @return height (in px) of the template
	 */
	public int getHeight() {
		return height;
	}

	/**
	 * @return name of the template
	 */
	public String getName() {
		return name;
	}

	/**
	 * Overwrites the templates current name with the given name
	 * 
	 * @param name
	 */
	public void setName(String name) {
		this.name = name;
	}

	public void addCut(int x1, int y1, int x2, int y2) {
		// H2
		// Create new Line object with values x1, y1, x2, y2
		Line line = new Line(x1, y1, x2, y2);
		// Add the created line to lines list
		lines.add(line);
	}

	public Line getCut(int i) {
		// H2
		// Get line from list
		return lines.get(i);
	}

	public int numberOfCuts() {
		// H2
		// Get size of lines list
		return lines.size();
	}

	public static CuttingTemplate parseCuttingTemplate(String[] lines) {
		// H3
		// Get Width, Height (Breite, Hhe) from first entry from lines
		// Split String by ";" to Array (The first entry on new array is Width and second entry is Height)
		String[] splittedFirstLine = lines[0].split(";");
		// Convert String to Integer
		int width = Integer.parseInt(splittedFirstLine[0]);
		int height = Integer.parseInt(splittedFirstLine[1]);
		
		// Create new CuttingTemplate
		CuttingTemplate ct = new CuttingTemplate(width, height);
		
		// Add lines to CuttingTemplate // Ignore first entry in Array! (because its not a line)
		for (int lineNumber = 1; lineNumber < lines.length; lineNumber++) {
			String line = lines[lineNumber];
			
			// Split line by "->" to get First Points (x1,y1) and Second Point (x2,y2)
			String[] splittedLine = line.split("->");
			// Split splittedLine[0] (First Points) by ":" to get x1 and y1
			String[] firstPoints = splittedLine[0].split(";");
			// Split splittedLine[1] (Second Points) by ":" to get x2 and y2
			String[] secondPoints = splittedLine[1].split(";");
			
			// Convert String to Integer
			int x1 = Integer.parseInt(firstPoints[0]);
			int y1 = Integer.parseInt(firstPoints[1]);
			
			int x2 = Integer.parseInt(secondPoints[0]);
			int y2 = Integer.parseInt(secondPoints[1]);
			
			// Add line to CuttingTemplate
			ct.addCut(x1, y1, x2, y2);
		}
		
		return ct;
	}

	public BufferedImage drawTemplate(Function<Line, Color> color) {
		BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
		Graphics g = image.getGraphics();

		// H5
		// Draw Lines
		for (int i = 0; i < this.lines.size(); i++) {
			Line line = this.lines.get(i);
			g.setColor(color.apply(line));
			g.drawLine(line.x1, line.y1, line.x2, line.y2);
		}
		
		// Draw Border (Rect)
		// Set Color to Black
		g.setColor(Color.BLACK);
		// Draw the border as rect on position x = 0; y = 0
		// Attention: The end of rect is on x + width / y + height
		// This means by width 100 it would start on position 0 and end position 100
		// But this would be a size of 101! You need to reduce by 1.
		g.drawRect(0, 0, width - 1, height - 1);

		return image;
	}

}
